import { ExactType, ExactTypeString } from "type-tls";
export declare type StringifyReviver = (this: any, key: string, value: any, type: string, callCount: number, stringifyOptions: StringifyReviverOptions) => any;
export declare type ParseReviver = (this: any, key: string, value: any, type: string, callCount: number) => any;
export declare type SPReviver = (this: any, key: string, value: any, type: string, callCount: number, stringifyOptions: StringifyReviverOptions | undefined) => any;
export interface ReviverPair {
    string: StringifyReviver;
    parse: ParseReviver;
}
export declare type Reviver = SPReviver | StringifyReviver | ParseReviver | ReviverPair;
export declare type DataType = ExactTypeString | ExactType;
export declare type DataTypeArray = DataType[];
export declare type Types = DataType | DataTypeArray;
export declare type TypeReviverArray<Revr> = [Types, Revr][];
export declare type TypeStringReviverArray<Revr> = [ExactTypeString | ExactTypeString[], Revr][];
export declare type TypeStringReviverFlatArray<Revr> = [ExactTypeString, Revr][];
/**
 * 扁平化的 TypeReviverArray
 */
export declare type TypeReviverFlatArray<Revr> = [DataType, Revr][];
export interface TypeReviverObject<Revr> {
    [typeName: string]: Revr;
}
export declare type TypeReviverMap<Revr> = Map<Types, Revr>;
/**
 * 扁平化的 TypeReviverMap
 */
export declare type TypeReviverFlatMap<Revr> = Map<DataType, Revr>;
export declare function isReviverPair(target: any): target is ReviverPair;
export declare type TypeRevivers<Revr> = TypeReviverArray<Revr> | TypeReviverObject<Revr> | TypeReviverMap<Revr>;
/**
 * 将 typeRevivers 转成 TypeReviverArray
 * @param typeRevivers
 */
export declare function toTypeReviverArray<Revr>(typeRevivers: TypeRevivers<Revr>): TypeReviverArray<Revr>;
/**
 * 类型扁平化解析结果
 */
interface TypeFlatParseInfo<Revr> {
    typeFlat: TypeReviverFlatArray<Revr>;
    stringFlat: TypeStringReviverFlatArray<Revr>;
    trObject: TypeReviverObject<Revr>;
    typeFun: Function[];
}
/**
 * 扁平化解析 TypeReviverArray
 * @param typeReviverArr
 */
export declare function flatParseTypeReviverArray<Revr>(typeReviverArr: TypeReviverArray<Revr>): TypeFlatParseInfo<Revr>;
/**
 * 扁平化解析 TypeRevivers
 * @param typeRevivers
 */
export declare function flatParseTypeRevivers<Revr>(typeRevivers: TypeRevivers<Revr>): TypeFlatParseInfo<Revr>;
export declare function toTypeStringReviverFlatArray<Revr>(typeRevivers: TypeRevivers<Revr>): TypeStringReviverFlatArray<Revr>;
export declare function toTypeReviverObject<Revr>(typeRevivers: TypeRevivers<Revr>): TypeReviverObject<Revr>;
export declare function toTypeReviverMap<Revr>(typeRevivers: TypeRevivers<Revr>): TypeReviverMap<Revr>;
export interface StringifyReviverOptions {
    skip?: boolean;
    skipMark?: boolean;
    skipRootMark?: boolean;
}
export interface JSONStringifyOptions extends StringifyReviverOptions {
    skipRoot?: boolean;
    space?: string | number;
    mark?: string | null;
}
/**
 * 自定义JSON序列化；可根据数据类型来自定义序列化方案； 给合  customJSONParse  可实现对任意类型的数据 进行序列化 并完整（无信息丢失）还原；
 * @param value: any   被序列化的对象
 * @param typeRevivers?:TypeRevivers|null    定义 类型 与 其对类的 自定义序列化函数；
 * @param options:JSONStringifyOptions    选项
 */
export declare function customJSONStringify<Revr extends Reviver>(value: any, typeRevivers?: TypeRevivers<Revr> | null, options?: JSONStringifyOptions): string;
export declare enum LostRevier {
    original = "original",
    parse = "parse",
    ignore = "ignore"
}
export interface JSONParseOptions {
    mark?: string | null;
    lostRevier?: LostRevier;
}
/**
 * 自定义JSON解析；可根据类型自定义解析逻辑；
 * @param text
 * @param typeRevivers
 * @param options
 */
export declare function customJSONParse<Revr extends Reviver>(text: string, typeRevivers?: TypeRevivers<Revr> | null, options?: JSONParseOptions): any;
export {};
